# frozen_string_literal: true

# app/hermes/models/concerns/common_field.rb
# app/controllers/concerns/test_concern.rb

module Hermes
    module Concerns
        module FormValidatorConcern
            extend ActiveSupport::Concern

            included do
                field :form_sne_validation_errors, type: Array, default: []
                field :form_front_validation_errors, type: Array, default: []

                around_save :validate_fields

                def validate_fields
                  before_validate_action
                    path = form_validator_csv_path
                    rule_to_checks = CSV.read(path)
                    version = rule_to_checks[0][1]
                    front_errors = []
                    sne_errors = []
                    rule_to_checks.each_with_index do |line_to_check,i|
                        if(i>1)
                            unless check_rule_to_check_line_to_check(line_to_check)
                                if line_to_check[6] == "1"
                                    sne_errors.push(format_error(line_to_check, i, version))
                                else
                                    front_errors.push(format_error(line_to_check, i, version))
                                end
                            end
                        end
                    end
                   
                    self.form_front_validation_errors = front_errors
                    self.form_sne_validation_errors = sne_errors
                    after_validate_action
                    yield
                end

                def after_validate_action
                end 
                def before_validate_action
                end 

                def check_rule_to_check_line_to_check(line_to_check)
                    if line_to_check[1].blank?
                        return true
                    else
                        verifified_only_if_condition = line_to_check[2]
                        case verifified_only_if_condition
                        when "none", nil, ""
                            verify_rule_to_check(line_to_check)
                        when "custom_rule"
                          self.send(line_to_check[1])
                        else
                            validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                        end
                    end
                end

                def verify_rule_to_check(line_to_check)
                    rule_to_check = line_to_check[1]
                    field = line_to_check[0]
                    case rule_to_check
                    when "presence"
                         return validate_presence(field)
                    else
                        return validate_complex_rule_to_check(field, rule_to_check)
                    end
                end

                def validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                    begin
                        rule_to_check = line_to_check[1]
                        collected_validation_results = []
                        if(verifified_only_if_condition.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                if(eval(verifified_only_if_condition))
                                    case rule_to_check
                                    when "presence"
                                        collected_validation_results.push(validate_presence(line_to_check[0]))
                                    else
                                        collected_validation_results.push(eval(rule_to_check))
                                    end
                                   
                                end
                            end
                        else
                            if(eval(verifified_only_if_condition))
                                case rule_to_check
                                when "presence"
                                    collected_validation_results.push(validate_presence(line_to_check[0]))
                                else
                                    collected_validation_results.push(eval(rule_to_check))
                                end
                            end
                        end
                        
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with condition' 
                        ap line_to_check
                    end
                end
                def validate_presence(field)
                    array_values = get_values_from_object_path(field)|| []
                    array_values.any? { |val| val.is_a?(Boolean) ? !val.nil? : !val.blank?}
                end

                def validate_complex_rule_to_check(field, rule_to_check)
                    begin
                        collected_validation_results = []
                        if(rule_to_check.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                collected_validation_results.push(eval(rule_to_check))
                            end
                        else
                            collected_validation_results.push( eval(rule_to_check) )
                        end
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with rule_to_check' 
                        ap field
                    end
                end

                def get_values_from_object_path(field)
                    begin
                        array_path = field.split(".")
                        collected_values =[]
                        if(array_path[0] == "applicant")
                        
                            all_applicants_raw.each do |applicant|
                                    collected_values.push(eval(field))
                            end
                        else
                            collected_values.push(eval(field))
                        end
                        collected_values
                    rescue => e
                        ap e
                        ap 'error with field recuperation' 
                        ap field
                    end

                end

                def format_alin_field(label)
                    minus_two_year = main_applicant.get_minus_two_year
                    label.sub! "[tax-year-0]" , (minus_two_year + 2).to_s
                    label.sub! "[tax-year-1]" , (minus_two_year + 1).to_s
                    label.sub! "[tax-year-2]" , (minus_two_year).to_s
                    label
                end
                def format_error(line_to_check, i, version)
                    {
                        "category" => line_to_check[3],
                        "sub_category" => line_to_check[4],      
                        "alin_field" => format_alin_field(line_to_check[5]),       
                        "csv_rule_to_check_line_to_check" => (i+1).to_s,                
                        "hermes_field" => line_to_check[0],     
                        "rule_to_check" => line_to_check[1],                
                        "verification_condition" =>line_to_check[2] ,     
                        "csv_rule_to_checks_version" => version,
                        "is_sne" => line_to_check[6]          
                    }
                end

                def check_employer_name
                  app =  get_applicant_used_for_siret
                  if app&.work_contract_type.in?(['CDI', 'CDD',  'CDIP', 'APP', 'AGP'])
                    return app.main_employer_name.present?
                  end
                  true
                end

                def check_consent_level
                  app =  get_applicant_used_for_siret
                  if entity_id && app&.work_contract_type.in?(['CDI', 'CDD',  'CDIP', 'APP', 'AGP'])
                    return !!consent_level
                  end 
                  true
                end 
            end
        end
    end
end
